using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace CBXInput4XNA_Example1
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        CBXInput4XNA.IUserInputService input;
        IList<string> items = new List<string>();
        SpriteFont font;

        
        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            this.IsMouseVisible = true;
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // create the instance of the user input class
            this.input = new CBXInput4XNA.UserInputService(this);
            
            // add it to the services list so other code can gain access to it
            this.Services.AddService(this.input.GetType(), this.input);
            
            // add it as a game component. Adding it as a game component allows the update method 
            // to automatically be called each time the game loop runs
            this.Components.Add((IGameComponent)this.input);

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // create a font from the game asset
            this.font = this.Content.Load<SpriteFont>("CourierNew");
            
            // TODO: use this.Content to load your game content here
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // check if left or right mouse buttons were clicked and if so add it to the list.
            if (this.input.LeftMouseButtonClicked) this.items.Add("Left mouse button clicked");
            if (this.input.RightMouseButtonClicked) this.items.Add("Right mouse button clicked");

            // check if player ones game pad A button states
            if (this.input.A(PlayerIndex.One) == ButtonState.Pressed && 
                this.input.PreviousA(PlayerIndex.One) == ButtonState.Released) this.items.Add("Player One A button is pressed");
            if (this.input.A(PlayerIndex.One) == ButtonState.Released &&
                this.input.PreviousA(PlayerIndex.One) == ButtonState.Pressed) this.items.Add("Player One A button is released");
            if (this.input.AClick(PlayerIndex.One)) this.items.Add("Player One A button clicked");
                     
            // check keyboard arrow key states
            if (this.input.KeyClick(PlayerIndex.One, Keys.Up)) this.items.Add("Player One Up KB button was clicked");
            if (this.input.KeyClick(PlayerIndex.One, Keys.Down)) this.items.Add("Player One Down KB button was clicked");
            if (this.input.KeyClick(PlayerIndex.One, Keys.Left)) this.items.Add("Player One Left KB button was clicked");
            if (this.input.KeyClick(PlayerIndex.One, Keys.Right)) this.items.Add("Player One Right KB button was clicked");

            // limit list to 20 items
            while (this.items.Count > 20)     
            {
                // this is a test
                items.RemoveAt(0);
            }

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            this.spriteBatch.Begin();
            // draw the list
            Vector2 pos = Vector2.Zero;
            foreach (string str in this.items)
            {
                this.spriteBatch.DrawString(this.font, str, pos, Color.Red);
                pos.Y += this.font.LineSpacing;
            }

            this.spriteBatch.End();
            base.Draw(gameTime);
        }
    }
}
